module Polynomial where

import Data.List
import Data.Maybe

-- linear polynomial
-- constant part is associated with empty string ""
type Polynomial = [(String, Int)]

var :: String -> Polynomial
var x = [(x,1)]

num :: Int -> Polynomial
num n = [("",n)]

normalize :: Polynomial -> Polynomial
normalize []           = []
normalize ((x, m) : v) =
  (x, m + sum [ n | (y, n) <- v, x == y ]) : 
  normalize [ (y, n) | (y, n) <- v, x /= y ]

plus :: [Polynomial] -> Polynomial
plus vs = normalize (concat vs)

times :: Int -> Polynomial -> Polynomial
times k v = [ (x, k * n) | (x, n) <- v ]

coefficient :: String -> Polynomial -> Int
coefficient x v = fromMaybe 0 (lookup x v)

variables :: Polynomial -> [String]
variables v = [ x | (x, _) <- v ]

geq :: Polynomial -> Polynomial -> Bool
geq v1 v2 = all p (union (variables v1) (variables v2))
  where
    p x = coefficient x v1 >= coefficient x v2
 
gt :: Polynomial -> Polynomial -> Bool
gt v1 v2 = all p (union (variables v1) (variables v2))
  where
    p "" = coefficient "" v1 >  coefficient "" v2
    p x  = coefficient x  v1 >= coefficient x  v2
