module Recursive where

import TRS
import qualified EncodingLinear1
import qualified EncodingLinear2
import qualified EncodingLinear3
import qualified EncodingMaxPlus


data CriterionProof = 
    Linear1Proof EncodingLinear1.Algebra
  | Linear2Proof EncodingLinear2.Algebras
  | Linear3Proof EncodingLinear3.Algebras
  | MaxPlusProof EncodingMaxPlus.Algebra

type Criterion = TRS -> TRS -> IO (Maybe (CriterionProof, TRS))

type Proof = [(CriterionProof, TRS)]

instance Show CriterionProof where
  show :: CriterionProof -> String
  show (Linear1Proof a) = EncodingLinear1.showAlgebra  a
  show (Linear2Proof a) = EncodingLinear2.showAlgebras a
  show (Linear3Proof a) = EncodingLinear3.showAlgebras a
  show (MaxPlusProof a) = EncodingMaxPlus.showAlgebra  a

showProof :: Proof -> String
showProof [] = 
  "No dependency pair remains.\n" 
showProof ((method, removed) : pairs) =
  show method ++ "\n" ++
  "strictly orients the following dependency pairs:\n\n" ++
  showTRS removed ++ "\n" ++
  "We remove them from the set of dependency pairs.\n\n" ++
  showProof pairs

linearAll :: String -> Criterion
linearAll tool trs1 trs2 = do
  result <- EncodingLinear2.solveAll tool trs1 trs2
  case result of
    Nothing            -> return Nothing
    Just (ab, removed) -> return (Just (Linear2Proof ab, removed))

linear1 :: String -> Criterion
linear1 tool trs1 trs2 = do
  result <- EncodingLinear1.solveOne tool trs1 trs2
  case result of
    Nothing           -> return Nothing
    Just (a, removed) -> return (Just (Linear1Proof a, removed))

linear2 :: String -> Criterion
linear2 tool trs1 trs2 = do
  result <- EncodingLinear2.solveOne tool trs1 trs2
  case result of
    Nothing           -> return Nothing
    Just (a, removed) -> return (Just (Linear2Proof a, removed))

linear3 :: String -> Criterion
linear3 tool trs1 trs2 = do
  result <- EncodingLinear3.solveOne tool trs1 trs2
  case result of
    Nothing            -> return Nothing
    Just (abc, removed) -> return (Just (Linear3Proof abc, removed))

maxplus :: String -> Criterion
maxplus tool trs1 trs2 = do
  result <- EncodingMaxPlus.solveOne tool trs1 trs2
  case result of
    Nothing            -> return Nothing
    Just (ab, removed) -> return (Just (MaxPlusProof ab, removed))

solveOne :: [Criterion] -> TRS -> TRS -> IO (Maybe (CriterionProof, TRS))
solveOne [] _ _ = return Nothing
solveOne (criterion : criteria) trs1 trs2 = do
    result1 <- criterion trs1 trs2
    case result1 of
      Nothing    -> solveOne criteria trs1 trs2
      Just proof -> return (Just proof)

solve :: [Criterion] -> TRS -> TRS -> IO (Maybe [(CriterionProof,TRS)])
solve _    []   _    = return (Just []) 
solve criteria trs1 trs2 = do
  result1 <- solveOne criteria trs1 trs2
  case result1 of
    Nothing              -> return Nothing
    Just (pair, removed) -> do
      let trs1' = [ rule | rule <- trs1, notElem rule removed ]  
      result2 <- solve criteria trs1' trs2
      case result2 of
        Nothing    -> return Nothing
        Just pairs -> return (Just ((pair, removed) : pairs))