module RelativeDP where

import Term
import Rule
import TRS

isHeadedByDefined :: [String] -> Term -> Bool
isHeadedByDefined ds (F f _) = elem f ds
isHeadedByDefined _ _ = False

dpFromRule :: [String] -> Rule -> TRS
dpFromRule ds (l, r) =
  [ Rule.mark (l, t) | t <- subterms r, isHeadedByDefined ds t ]

dp :: [String] -> TRS -> TRS
dp ds trs = [ p | rule <- trs, p <- dpFromRule ds rule ] 

dominates' :: TRS -> Rule -> Bool
trs `dominates'` (_, r) =
  all (\f -> not (elem f (definedSymbols trs))) (Term.functions r)

dominates :: TRS -> TRS -> Bool
trs1 `dominates` trs2 = all (dominates' trs1) trs2

almostDominates' :: TRS -> Rule -> Bool
trs `almostDominates'` rule@(l@(F f _), r) = trs `dominates'` rule ||
  case flatten ds r of
    r'@(F g _) -> f == g && elem f ds && shallow l && shallow r'
    _ -> False
  where
    ds = definedSymbols trs
trs `almostDominates'` rule = trs `dominates'` rule

almostDominates :: TRS -> TRS -> Bool
trs1 `almostDominates` trs2 = all (almostDominates' trs1) trs2

flatten :: [String] -> Term -> Term
flatten _ x@(V _) = x
flatten ds (F f ts)
  | elem f ds = F f (map (flatten ds) ts)
  | [u] <- us = u
  | otherwise = F f us
  where
    us = [ u | t <- ts, u <- flatten_list ds t ]

-- looking at below a constructor symbol
flatten_list :: [String] -> Term -> [Term]
flatten_list ds t = case flatten ds t of
  F g ts | not (elem g ds) -> ts
  t' -> [t']

-- >>> flatten ["f"] (F "g" [F "a" [], F "f" [F "h" [V "y"], V "x"]])
-- f(y,x)
--
